# $Revision: 1.2 $ $Date: 2000/06/09 03:50:40 $
# Pamela Whitfield & Brian Toby
# a GUI for March-Dollase preferred orientatio

# get March-Dollase preferred orientation information
# use MDprefinfo hist phase axis-number parm action value
#    ratio    -- ratio of xtallites in PO direction vs random (>1 for more)
#    fraction -- fraction in this direction, when more than one axis is used
#    h k & l  -- indices of P.O. axis
#    ratioref -- flag to vary ratio
#    fracref  -- flag to vary fraction
#    damp     -- damping value
#    type     -- model type (0 = P.O. _|_ to beam, 1 = || to beam)
#    new      -- creates a new record with default values (set only)
proc MDprefinfo {histlist phaselist axislist parm "action get" "value {}"} {
    foreach phase $phaselist hist $histlist axis $axislist {
	if {$phase == ""} {set phase [lindex $phaselist end]}
	if {$hist == ""} {set hist [lindex $histlist end]}
	if {$axis == ""} {set axis [lindex $axislist end]}
	if {$hist < 10} {
	    set hist " $hist"
	}
	if {$axis > 9} {
	    set axis "0"
	}
	set key "HAP${phase}${hist}PREFO${axis}"
	switch -glob ${parm}-$action {
	    ratio-get {
 		return [string trim [string range [readexp $key] 0 9]]
	    }
	    ratio-set {
		if ![validreal value 10 6] {return 0}
		setexp $key $value 1 10
	    }
	    fraction-get {
 		return [string trim [string range [readexp $key] 10 19]]
	    }
	    fraction-set {
		if ![validreal value 10 6] {return 0}
		setexp $key $value 11 10
	    }
	    h-get {
		set h [string trim [string range [readexp $key] 20 29]]
		# why not allow negative h values?
		#		if {$h < 1} {return 0}
		return $h
	    }
	    h-set {
		if ![validreal value 10 2] {return 0}
		setexp $key $value 21 10
	    }
	    k-get {
		set k [string trim [string range [readexp $key] 30 39]]
		#		if {$k < 1} {return 0} 
		return $k
	    }
	    k-set {
		if ![validreal value 10 2] {return 0}
		setexp $key $value 31 10
	    }
	    l-get {
		set l [string trim [string range [readexp $key] 40 49]]
		#if {$l < 1} {return 0}
		return $l
	    }
	    l-set {
		if ![validreal value 10 2] {return 0}
		setexp $key $value 41 10
	    }
	    ratioref-get {
		if {[string toupper \
			[string range [readexp $key] 53 53]] == "Y"} {
		    return 1
		}
		return 0
	    }
	    ratioref-set {
		if $value {
		    setexp $key "Y" 54 1
		} else {
		    setexp $key "N" 54 1
		}
	    }
	    fracref-get {
		if {[string toupper \
			[string range [readexp $key] 54 54]] == "Y"} {
		    return 1
		}
		return 0
	    }
	    fracref-set {
		if $value {
		    setexp $key "Y" 55 1
		} else {
		    setexp $key "N" 55 1
	      }
	    }
	    damp-get {
		set val [string trim [string range [readexp $key] 59 59]]
		if {$val == " "} {return 0}
		return $val
	    }
	    damp-set {
		setexp $key $value 60 1
	    }
	    type-get {
		set val [string trim [string range [readexp $key] 64 64]]
		if {$val == " "} {return 0}
		return $val
	    }
	    type-set {
		# only valid settings are 0 & 1
		if {$value != "0" && $value != "1"} {set value "0"}
		setexp $key $value 65 1
	    }
	    new-set {
		makeexprec $key
		setexp $key \
			{  1.000000  1.000000  0.000000  0.000000  1.000000   NN    0    0} \
			1 68
	    }
	    default {
		set msg "Unsupported MDprefinfo access: parm=$parm action=$action"
		tk_dialog .badexp "Error in EXP" $msg error 0 Exit 
		destroy .
	    }

	}

    }
}

proc MakeOrientPane {} {
    global expgui expmap
    pack [frame $expgui(orientFrame).hs] -side left -expand y -fill both
    grid [listbox $expgui(orientFrame).hs.title -height 1 -relief flat \
	    -exportselection 0 \
	    -font $expgui(histfont) ] -row 0 -column 0 -sticky ew
    grid [listbox $expgui(orientFrame).hs.lbox -height 10 -width 25 \
	    -exportselection 0 \
	    -font $expgui(histfont) \
	    -xscrollcommand "$expgui(orientFrame).hs.x set" \
	    -yscrollcommand "$expgui(orientFrame).hs.y set" \
	    ] -row 1 -column 0 -sticky news
    # register the listbox so it gets updated
    lappend expgui(HistSelectList) $expgui(orientFrame).hs
    grid [scrollbar $expgui(orientFrame).hs.x -orient horizontal \
	    -command "move2boxes \" $expgui(orientFrame).hs.title $expgui(orientFrame).hs.lbox \" " 
    ] -row 2 -column 0 -sticky ew
    grid [scrollbar $expgui(orientFrame).hs.y \
	    -command "$expgui(orientFrame).hs.lbox yview"] \
	    -row 1 -column 1 -sticky ns
    grid columnconfigure $expgui(orientFrame).hs 0 -weight 1
    grid rowconfigure $expgui(orientFrame).hs 1 -weight 1
    bind $expgui(orientFrame).hs.lbox <ButtonRelease-1> {
	set expgui(curhist) [$expgui(orientFrame).hs.lbox curselection]
	DisplayOrient
    }
    bind $expgui(orientFrame).hs.lbox <Button-3>  {
	if $expgui(globalmode) {
	    $expgui(orientFrame).hs.lbox selection set 0 end
	    set expgui(curhist) [$expgui(orientFrame).hs.lbox curselection]
	    DisplayOrient
	}
    }
    # Create a frame on the right side
    if $expgui(haveBW) {
	pack [TitleFrame $expgui(orientFrame).f1 -bd 4 \
		-text "March-Dollase Preferential Orientation" \
		-relief groove] -fill both -expand true
	set PrefOrientBox [$expgui(orientFrame).f1 getframe]
    } else {
	pack [frame $expgui(orientFrame).f1] -fill both -expand true
	set PrefOrientBox $expgui(orientFrame).f1
    }
    grid columnconfigure $PrefOrientBox 0 -weight 1
    grid rowconfigure $PrefOrientBox 1 -weight 1
    # Create canvas with a frame inside for scrolling
    grid [set expgui(OrientBox) [canvas $PrefOrientBox.orientBox \
	    -scrollregion {0 0 5000 500} \
	    -yscrollcommand "$PrefOrientBox.yscroll set" \
	    -width 500 -height 350 -bg lightgrey]] \
	    -sticky news -row 1 -column 0
    set expgui(OrientScroll) [scrollbar $PrefOrientBox.yscroll \
	    -command "$expgui(OrientBox) yview" \
	    -orient vertical]
    # control the griding of the scrollbar in DisplayOrient
    #grid $PrefOrientBox.yscroll -sticky ns -row 1 -column 1
    frame $expgui(OrientBox).f -bd 0	
    $expgui(OrientBox) create window 0 0 -anchor nw \
	    -window $expgui(OrientBox).f 

    # insert the histograms & resize in case the pane needs more space
    sethistlist
#    ResizeNotebook
}

# this is used to update the contents of the PO page when histogram(s)
# are selected
proc DisplayOrient {} {
    global expgui entrycmd entryvar expmap
	
    # identify the frame and kill the old contents
    set pOrientf1 $expgui(OrientBox).f
    eval destroy [winfo children $pOrientf1]
    grid columnconfig $pOrientf1 0 -weight 1
    grid columnconfig $pOrientf1 15 -weight 1
    grid columnconfig $pOrientf1 9 -min 10
    grid columnconfig $pOrientf1 12 -min 10
    # trap if more than one histogram is selected unless global mode
    if {$expgui(globalmode) == 0 && [llength $expgui(curhist)] > 1} {
	set expgui(curhist) [lindex $expgui(curhist) 0]
    }

    # display the selected histograms
    $expgui(orientFrame).hs.lbox selection clear 0 end
    foreach h $expgui(curhist) {
	$expgui(orientFrame).hs.lbox selection set $h
    }

    #disable traces on entryvar
    set entrycmd(trace) 0
    trace vdelete entryvar w entvartrace
    
    #display selected histograms
    $expgui(orientFrame).hs.lbox selection clear 0 end
    foreach hist $expgui(curhist) {
	$expgui(orientFrame).hs.lbox selection set $hist
    }

    #get histogram list by histogram number
    set histlist {}
    foreach item $expgui(curhist) {
	lappend histlist [lindex $expmap(powderlist) $item]
    }

    # loop over histograms and phases
    set row -1
    foreach hist $histlist {
	foreach phase $expmap(phaselist_$hist) {
	    grid [frame $pOrientf1.sp$row -bd 8 -bg white] \
		-columnspan 20 -column 0 -row [incr row] -sticky nsew
	    # add extra label here when more than one histogram is selected
	    if {[llength $histlist] > 1} {
		set lbl "Histogram $hist\nPhase $phase"
	    } else {
		set lbl "Phase $phase"
	    }
	    grid [label $pOrientf1.l1$row -text $lbl] \
		    -column 0 -row [incr row] -sticky nws
	    set naxis [hapinfo $hist $phase POnaxis]
	    set col 0
	    foreach var {h k l} {
		grid [label $pOrientf1.l${var}$row -text $var \
			-anchor center] \
			-column [incr col] -row $row -sticky ews
	    }
	    grid [label $pOrientf1.lrat$row -text "Ratio" -anchor center] \
		    -column 10 -row $row -sticky ews
	    if {$naxis > 1} {
		grid [label $pOrientf1.lfrac$row -text "Fraction" \
			-anchor center] \
			-column 13 -row $row -sticky ews
	    }
	    grid [label $pOrientf1.ld$row -text "Damping"] \
		    -column 15 -row $row -sticky es
	    for {set axis 1} {$axis <= $naxis} {incr axis} {
		set phax ${phase}_$axis
		# define variables needed
		foreach var {ratio fraction ratioref fracref damp type} {
		    set entrycmd(${var}$phax) \
			    "MDprefinfo $hist $phase $axis $var"
		    set entryvar(${var}$phax) [eval $entrycmd(${var}$phax)]
		}
		foreach var {h k l} {
		    set entrycmd(${var}$phax) \
			    "MDprefinfo $hist $phase $axis $var"
		    set entryvar(${var}$phax) \
			    [format %.2f [eval $entrycmd(${var}$phax)]]
		}
		incr row
		set col -1
		grid [label $pOrientf1.axis$row -text "axis $axis"\
			-anchor center ] \
			-column [incr col] -row $row
		set col 0
		# Axis
		foreach var {h k l} {
		    grid [entry $pOrientf1.e${var}$row \
			    -textvariable entryvar(${var}$phax) -width 4] \
			    -column [incr col] -row $row
		}
		# Ratio
		grid [entry $pOrientf1.erat$row \
			-textvariable entryvar(ratio$phax) -width 10] \
			-column 10 -row $row -sticky e
		# ratio refine
		grid [checkbutton $pOrientf1.ratref$row \
			-variable entryvar(ratioref$phax)] \
			-column 11 -row $row -sticky w
		if {$naxis > 1} {
		    # Fraction
		    grid [entry $pOrientf1.efrac$row \
			    -textvariable entryvar(fraction$phax) -width 10] \
			    -column 13 -row $row -sticky e
		    # fraction refine
		    grid [checkbutton $pOrientf1.fracref$row \
			    -variable entryvar(fracref$phax)] \
			    -column 14 -row $row -sticky w
		}
  		#damp
		tk_optionMenu $pOrientf1.opd$row \
			entryvar(damp$phax) \
			0 1 2 3 4 5 6 7 8 9
		grid $pOrientf1.opd$row \
			-column 15 -row $row -sticky e
	    }
	    grid [button $pOrientf1.add$row -text "Add axis" \
		    -command "AddNewPOaxis $hist $phase"] \
		    -column 0 -columnspan 4 -row [incr row] -sticky nws

	}
    }
    grid [frame $pOrientf1.sp$row -bd 8 -bg white] \
	    -columnspan 20 -column 0 -row [incr row] -sticky nsew

    # resize the scroll area
    update
    set sizes [grid bbox $pOrientf1]
    $expgui(OrientBox) config -scrollregion $sizes -width [lindex $sizes 2]
    # use the scroll for BIG lists
    if {[lindex $sizes 3] > [winfo height $expgui(OrientBox)]} {
	grid $expgui(OrientScroll) -sticky ns -column 1 -row 1
    } else {
	grid forget $expgui(OrientScroll)
    }
    update
    #enable traces on entryvar now
    set entrycmd(trace) 1
    trace variable entryvar w entvartrace
    ResizeNotebook
}

proc AddNewPOaxis {hist phase} {
    global expgui
    set nextaxis [hapinfo $hist $phase POnaxis]
    incr nextaxis
    if {$nextaxis > 9} return
    MDprefinfo $hist $phase $nextaxis new set
    hapinfo $hist $phase POnaxis set $nextaxis
    incr expgui(changed)
    DisplayOrient
}
